package com.xydl.util;

import lombok.extern.slf4j.Slf4j;
import org.eclipse.paho.client.mqttv3.*;
import org.eclipse.paho.client.mqttv3.persist.MemoryPersistence;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import java.text.SimpleDateFormat;
import java.util.Date;

@Component
@Slf4j
public class MqttUtil {

    @Value("${mqtt.broker}")
    private String broker;

    @Value("${mqtt.topic}")
    private String topic;

    @Value("${mqtt.username}")
    private String username;

    @Value("${mqtt.password}")
    private String password;

    @Value("${mqtt.publish.clientid}")
    private String publishClientid;

    @Value("${mqtt.subscribe.clientid}")
    private String subscribeClientid;

    @Value("${mqtt.qos}")
    private int qos;


    public boolean publish2MQTT(String content) {
        MqttClient client;
        MqttDeliveryToken token;
        try {
            client = new MqttClient(broker, publishClientid, new MemoryPersistence());
        } catch (MqttException e) {
            throw new RuntimeException(e);
        }
        MqttConnectOptions options = new MqttConnectOptions();
        options.setUserName(username);
        options.setPassword(password.toCharArray());
        options.setConnectionTimeout(60);
        options.setKeepAliveInterval(30);
        // connect
        try {
            client.connect(options);
        } catch (MqttException e) {
            throw new RuntimeException(e);
        }
        // create message and setup QoS
        MqttMessage message = new MqttMessage(content.getBytes());
        message.setQos(qos);
        // publish message
        try {
            token = client.getTopic(topic).publish(message);
            token.waitForCompletion();
            //打印发送状态
//            log.info("messagei is published completely! {}", token.isComplete());
//            client.publish(topic, message);
        } catch (MqttException e) {
            throw new RuntimeException(e);
        }
        // disconnect
        try {
            client.disconnect();
        } catch (MqttException e) {
            throw new RuntimeException(e);
        }
        // close client
        try {
            client.close();
        } catch (MqttException e) {
            throw new RuntimeException(e);
        }

        return token.isComplete();
    }

    public void subScribeMQTT() {

//        String broker = "tcp://139.196.211.222:10883";
//        String topic = "mqtt/test";
//        String username = "test";
//        String password = "AliOS%1688";
//        String clientid = "subscribe_client";
//        int qos = 0;
        try {
            MqttClient client = new MqttClient(broker, subscribeClientid, new MemoryPersistence());
            // connect options
            MqttConnectOptions options = new MqttConnectOptions();
            options.setUserName(username);
            options.setPassword(password.toCharArray());
            options.setConnectionTimeout(60);
            options.setKeepAliveInterval(30);
            // setup callback
            client.setCallback(new MqttCallback() {
                public void connectionLost(Throwable cause) {
                    log.info("connectionLost:{}", cause.getMessage());
                }

                public void messageArrived(String topic, MqttMessage message) {
                    log.info("topic:{} ", topic);
                    log.info("Qos:{} ", message.getQos());
                    log.info("{}", new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(new Date()) + " message content接收: " + new String(message.getPayload()));

                }

                public void deliveryComplete(IMqttDeliveryToken token) {
                    log.info("deliveryComplete---------{}", token.isComplete());
                }
            });
            client.connect(options);
            client.subscribe(topic, qos);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }


}
